//FILE: main.cpp
//PURPOSE: open a library, get and instance a class, use the class
//AUTHOR: Gian Paolo Ghilardi, a.k.a. rejected || jp
//MAIL: (jp80@tin.it)

//TO COMPILE: g++ main.cpp -o testlib.app -ldl

#include "lgomd.hpp"
#include <iostream>
#include <stdlib.h>
#include <dlfcn.h>


#define EXIT_SUCCESS 0
#define EXIT_FAILURE 1

using namespace std;

int main( int argc, char **argv)
{
	//if no value was inserted at the command line, we try to get the load of
	//a node, i.e. the 7th.
	int nodeId = 7;

	/*
		1)Open the library (libgomd.so)
	*/
	cout << "- C++ CLASS LOADING EXAMPLE" << endl;
	cout << "- Trying to open the library..." << endl;
	void* handle = dlopen("./libgomd.so", RTLD_LAZY);
	if (!handle)
	{
		cerr << "- Cannot load library: " << dlerror() << endl;
		return EXIT_FAILURE;
	}


	/*
		2)Load symbols to create/destroy the instances of libgomd classes
	*/
	cout << "- Trying to load symbols from the library..." << endl;
	create_t* create_libgomd = (create_t*) dlsym(handle, "create");
	destroy_t* destroy_libgomd = (destroy_t*) dlsym(handle, "destroy");

	/*
		3)Manage symbols errors (if these symbols are not found,
		  an error will be raised)
	*/
	if (!create_libgomd || !destroy_libgomd)
	{
		cerr << "- Cannot load symbols: " << dlerror() << endl;
		return EXIT_FAILURE;
	}

	/*
		4)All went right: print a message.
	*/
	cout << "- Ok: all symbols were found and loaded successfully!!!" << endl;


	/*
		5)Now we create an instance of the class libgomd
	*/
	lgomd* lg = create_libgomd();

	/*
		6)Now we get the load of a specific node
		Please notice the different approach:
		- m_id is included in lgomd.hpp
		- getLoad() is implemented in libgomd.cpp
	*/
	cout << "- This is libgomd instance #" << lg->m_id << endl;
	if(argc > 1)
	{
		//if inserted,the first param of command line is "our" nodeID
		nodeId = atoi(argv[1]);
	}
	lg->getLoad( nodeId ); //NOTE: getLoad( nodeID ) prints the value of a nodeId!
	cout << "- Ok: exiting..." << endl;
	return (EXIT_SUCCESS);


}
